from __future__ import absolute_import

import contextlib
from pip.utils.logging import get_indentation, _log_state


@contextlib.contextmanager
def indent_log(num=2):
    """
    I submited PR to pip because this has raising exception for us

    https://github.com/pypa/pip/pull/3161

    .. code-block:: python

        Exception:
        Traceback (most recent call last):
          File "/srv/leonardo/sites/majklk/local/lib/python2.7/site-packages/pip/basecommand.py", line 211, in main
            status = self.run(options, args)
          File "/srv/leonardo/sites/majklk/local/lib/python2.7/site-packages/pip/commands/install.py", line 344, in run
            requirement_set.cleanup_files()
          File "/srv/leonardo/sites/majklk/local/lib/python2.7/site-packages/pip/req/req_set.py", line 589, in cleanup_files
            with indent_log():
          File "/usr/lib/python2.7/contextlib.py", line 17, in __enter__
            return self.gen.next()
          File "/srv/leonardo/sites/majklk/local/lib/python2.7/site-packages/pip/utils/logging.py", line 38, in indent_log
            _log_state.indentation -= num
        AttributeError: 'thread._local' object has no attribute 'indentation'
    """
    _log_state.indentation = get_indentation() + num
    try:
        yield
    finally:
        _log_state.indentation = get_indentation() - num




from django.utils.translation import ugettext_lazy as _
from django_select2.forms import Select2Widget
from leonardo import forms, messages

from .utils import pip_install, update_all


class PluginInstallForm(forms.SelfHandlingForm):

    """simple form for installing packages

    this support new abilities like an dynamic plugin install etc..
    """

    packages = forms.ChoiceField(label=_('Search packages'), widget=Select2Widget())

    reload_server = forms.BooleanField(
        label=_('Reload Server'), initial=False,
        required=False,
        help_text=_('Warning: this kill this Leonardo instance !!!\
                    For successfull reload must be run under Supervisor !\
                    You may lost your data !'),)

    def __init__(self, *args, **kwargs):
        kwargs.pop('request', None)
        super(PluginInstallForm, self).__init__(*args, **kwargs)

        # I think that this is not best solution
        self.fields['packages'].choices = [
            (repo.name, repo.name, )
            for repo in update_all()]

        self.helper.layout = forms.Layout(
            forms.TabHolder(
                forms.Tab('Main',
                          'packages',
                          css_id='plugins-install-main'
                          ),
                forms.Tab('Advance',
                          'reload_server'
                          )
            )
        )

    def handle(self, request, data):

        kwargs = data
        kwargs['request'] = request
        try:
            pip_install(**kwargs)
        except Exception as e:
            messages.error(request, str(e))
        else:
            return True
        return False



import os

import django.dispatch
from django.conf import settings
from django.core import management
from django.utils import six
from django.utils.translation import ugettext_lazy as _
from django.views.debug import get_safe_settings
from horizon import tables
from horizon_contrib.tables import FilterAction
from leonardo import forms, leonardo, messages
from leonardo.utils import get_conf_from_module

from .tables import LeonardoTable, SettingsTable

server_restart = django.dispatch.Signal(providing_args=["request", "delay"])


def server_restart_callback(request, delay=0, **kwargs):

    # time.sleep(delay)
    try:

        # from django.utils.autoreload import restart_with_reloader, reloader_thread
        # kill self
        os.kill(os.getpid(), 9)

        messages.success(
            request, _('Server was successfuly restarted !'))
    except Exception as e:
        messages.error(request, str(e))
    else:
        return True
    return False


server_restart.connect(server_restart_callback)


class ServerReloadForm(forms.SelfHandlingForm):

    delay = forms.IntegerField(
        label=_('delay before restart'), initial=10, help_text=_('Delay before restart'))

    def handle(self, request, data):
        """PoC for self restart

        this support new abilities like an dynamic plugin install etc..
        """

        try:
            server_restart.send(sender=self.__class__,
                                request=request, delay=data['delay'])
            messages.warning(
                request, _('Server going to down !'))
        except Exception as e:
            messages.error(request, str(e))
        else:
            return True
        return False


class ManagementForm(forms.SelfHandlingForm):

    """form wich handle managemenet commands

    this supports running management commands via admin
    """

    makemigrations = forms.BooleanField(
        label=_('Make migrations'), initial=False,
        required=False,
        help_text=_('Run makemigrations after install ?'))

    migrate = forms.BooleanField(
        label=_('Migrate'), initial=False,
        required=False,
        help_text=_('Run migrate command after install ?'))

    sync_all = forms.BooleanField(
        label=_('Sync All'), initial=False,
        required=False,
        help_text=_('Run Sync All command after install ?'))

    sync_force = forms.BooleanField(
        label=_('Sync all force'), initial=False,
        required=False,
        help_text=_('Warning: this may override you database changes !'),)

    reload_server = forms.BooleanField(
        label=_('Reload Server'), initial=False,
        required=False,
        help_text=_('Warning: this kill this Leonardo instance !!!\
                    For successfull reload must be run under Supervisor !\
                    You may lost your data !'),)

    def __init__(self, *args, **kwargs):
        super(ManagementForm, self).__init__(*args, **kwargs)

        self.helper.layout = forms.Layout(
            forms.TabHolder(
                forms.Tab('Main',
                          'makemigrations',
                          'migrate',
                          'sync_all',
                          css_id='plugins-install-main'
                          ),
                forms.Tab('Advance',
                          'sync_force',
                          'reload_server',
                          )
            )
        )

    def handle(self, request, data):

        try:
            if data.get('makemigrations', None):
                management.call_command(
                    'makemigrations', verbosity=1, interactive=False)
            if data.get('migrate', None):
                management.call_command(
                    'migrate', verbosity=1, interactive=False)
            if data.get('sync_all', None):
                management.call_command(
                    'sync_all', force=data.get('sync_force', False))
            if data.get('reload_server', None):
                import os
                os.kill(os.getpid(), 9)
        except Exception as e:
            messages.error(request, str(e))
        else:
            return True
        return False


class InfoForm(forms.SelfHandlingForm):

    """wrapper for system info
    """

    def __init__(self, *args, **kwargs):
        request = kwargs.pop('request', None)
        super(InfoForm, self).__init__(*args, **kwargs)

        _settings = [{'key': k, 'value': v}
                     for k, v in six.iteritems(get_safe_settings())]

        table = SettingsTable(request, data=_settings)

        leonardo_table = LeonardoTable(request, data=leonardo.get_modules_as_list())

        self.helper.layout = forms.Layout(
            forms.TabHolder(
                forms.Tab('Leonardo modules',
                          forms.HTML(leonardo_table.render())
                          ),
                forms.Tab('Settings',
                          forms.HTML(table.render())
                          ),
            )
        )

    def handle(self, request, data):
        return True




from leonardo.utils.settings import dotdict

PIP_CONF = dotdict({
    'cache_dir': '/tmp',
    'trusted_hosts': ['https://pypi.python.org/pypi'],
    'index': 'https://pypi.python.org/pypi',
    'extra_index_urls': [],
    'no_index': [],
    'find_links:': []
})




from __future__ import absolute_import

from importlib import import_module
from leonardo.utils import is_leonardo_module
from pip.commands.list import ListCommand

LEONARDO_ENV = None
GLOBAL_ENV = None


def check_versions(only_leonardo=False):
    '''returns dictionary of modules with versions to could be updated

    return:: {
            'name': {
                        'old': '1.0.1',
                        'new': '1.0.2',
                        'type': wheel
                    }
            }
    '''

    global LEONARDO_ENV
    global GLOBAL_ENV

    if only_leonardo:
        if LEONARDO_ENV:
            return LEONARDO_ENV
    else:
        if GLOBAL_ENV:
            return GLOBAL_ENV

    listing = ListCommand()
    options, args = listing.parse_args([])

    update = {}

    for dist, version, typ in listing.find_packages_latest_versions(options):
        if only_leonardo:
            pkg_names = [k for k in dist._get_metadata("top_level.txt")]
            for pkg_name in pkg_names:
                try:
                    mod = import_module(pkg_name)
                except:
                    pass
                else:
                    if is_leonardo_module(mod):
                        if version > dist.parsed_version:
                            update.update({
                                dist.project_name: {
                                    'old': dist.version,
                                    'new': version,
                                    'type': typ
                                }
                            })
        else:
            if version > dist.parsed_version:
                update.update({
                    dist.project_name: {
                        'old': dist.version,
                        'new': version,
                        'type': typ
                    }
                })

    if only_leonardo:
        LEONARDO_ENV = update
    else:
        GLOBAL_ENV = update

    return update



from django.utils.translation import ugettext_lazy as _
from leonardo_admin_dashboard import modules


system_menu = modules.MenuModelList(
    _('Settings'),
    models=('constance.*', 'auth.*', 'sites.*', 'admin_sso.*',),
    order=99
)




from __future__ import absolute_import

from django.apps import AppConfig


default_app_config = 'leonardo_system.Config'


LEONARDO_APPS = ['leonardo_system']
LEONARDO_OPTGROUP = 'System'
LEONARDO_URLS_CONF = 'leonardo_system.urls'
LEONARDO_PAGE_ACTIONS = ['system/module_actions.html']
LEONARDO_ORDERING = 150
LEONARDO_DASHBOARD_MENU = ['leonardo_system.menu.system_menu']


class Config(AppConfig):
    name = 'leonardo_system'
    verbose_name = "Leonardo System Module"

    def ready(self):

        try:

            from leonardo_system.package.patch_pip import indent_log
            from pip.req import req_set
            req_set.indent_log = indent_log
        except:
            pass



import json
import os

import django.dispatch
from django.conf import settings
from django.core import management
from django.utils import six
from django.utils.translation import ugettext_lazy as _
from django.views.debug import get_safe_settings
from horizon import tables
from horizon_contrib.tables import FilterAction
from leonardo import forms, messages
from leonardo.utils import get_conf_from_module
from leonardo_system.management.commands._utils import get_versions
from django.template.loader import render_to_string
import pprint

pp = pprint.PrettyPrinter(indent=4)


class SettingsTable(tables.DataTable):

    key = tables.Column('key')
    value = tables.Column('value')

    def get_object_id(self, datum):
        return datum['key']

    class Meta:
        name = 'settings'
        table_actions = (FilterAction,)


class UninstallAction(tables.DeleteAction):

    data_type_singular = 'Module'
    data_type_plural = 'Modules'
    action_present = _("Uninstall")
    action_present_plural = _("Uninstalled")

    def allowed(self, request, instance=None):
        return True

    def action(self, request, obj_id):
        raise NotImplementedError


class UpgradeAction(tables.BatchAction):

    data_type_singular = 'Module'
    data_type_plural = 'Modules'
    action_present = _("Upgrade")
    action_present_plural = _("Upgraded")
    name = 'Upgrade'

    def allowed(self, request, instance=None):
        return True

    def action(self, request, obj_id):
        raise NotImplementedError


class LeonardoTable(tables.DataTable):

    name = tables.Column('module',
                         filters=(lambda m: m.__name__,))
    widgets = tables.Column(
        'widgets', verbose_name=_('Widgets'),
        filters=(lambda c: ', '.join([str(w.__name__) for w in c]),))
    plugins = tables.Column(
        'plugins', verbose_name=_('Plugins'),
        filters=(lambda c: ', '.join([p[0] for p in c]),))

    version = tables.Column('version',
                            verbose_name=_('Version'))

    needs_migrations = tables.Column('needs_migrations',
                                     verbose_name=_('Needs Migrations'),
                                     filters=(lambda c: _(
                                         'Needs Migrations') if c else '-',)
                                     )

    needs_sync = tables.Column('needs_sync',
                               verbose_name=_('Needs Sync'),
                               filters=(lambda c: _('Needs Sync') if c else '-',))

    def get_object_id(self, datum):
        return datum.name

    class Meta:
        name = 'leonardo-modules'
        verbose_name = _('Leonardo Modules')
        row_actions = (UpgradeAction, UninstallAction,)
        table_actions = (FilterAction,)
